@@ -79,6 +79,7 @@ static void groupwise_transfer_messages_to (CamelFolder *source, GPtrArray *uids
 static int gw_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args);
 void convert_to_calendar (EGwItem *item, char **str, int *len);
 static void convert_to_task (EGwItem *item, char **str, int *len);
+static void convert_to_note (EGwItem *item, char **str, int *len);
 static void gw_update_all_items ( CamelFolder *folder, GList *item_list, CamelException *ex);
 static void groupwise_populate_details_from_item (CamelMimeMessage *msg, EGwItem *item);
 static void groupwise_populate_msg_body_from_item (EGwConnection *cnc, CamelMultipart *multipart, EGwItem *item, char *body);
@@ -256,20 +257,23 @@ groupwise_populate_msg_body_from_item (EGwConnection *cnc, CamelMultipart *multi
 
 		case E_GW_ITEM_TYPE_APPOINTMENT:
 		case E_GW_ITEM_TYPE_TASK:
+		case E_GW_ITEM_TYPE_NOTE:
 			{
 				char *cal_buffer = NULL;
 				int len = 0;
 				if (type==E_GW_ITEM_TYPE_APPOINTMENT)
 					convert_to_calendar (item, &cal_buffer, &len);
-				else
+				else if (type == E_GW_ITEM_TYPE_TASK)
 					convert_to_task (item, &cal_buffer, &len);
+				else 
+					convert_to_note (item, &cal_buffer, &len);
+
 				camel_mime_part_set_content(part, cal_buffer, len, "text/calendar");
 				g_free (cal_buffer);
 				break;
 			}
 		case E_GW_ITEM_TYPE_NOTIFICATION:
 		case E_GW_ITEM_TYPE_MAIL:
-		case E_GW_ITEM_TYPE_NOTE:
 			if (body) 
 				camel_mime_part_set_content(part, body, strlen(body), "text/html");
 			else if (temp_body)
@@ -537,6 +541,7 @@ move_to_mailbox (CamelFolder *folder, CamelMessageInfo *info, CamelException *ex
 	g_ptr_array_add (uids, (gpointer) uid);
 
 	dest = camel_store_get_folder (folder->parent_store, "Mailbox", 0, ex);
+	camel_message_info_set_flags (info, CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_JUNK|CAMEL_MESSAGE_JUNK_LEARN|CAMEL_GW_MESSAGE_NOJUNK|CAMEL_GW_MESSAGE_JUNK, 0);
 	if (dest)
 		groupwise_transfer_messages_to (folder, uids, dest, NULL, TRUE, ex);
 	else
@@ -557,6 +562,7 @@ move_to_junk (CamelFolder *folder, CamelMessageInfo *info, CamelException *ex)
 	g_ptr_array_add (uids, (gpointer) uid);
 
 	dest = camel_store_get_folder (folder->parent_store, JUNK_FOLDER, 0, ex);
+	
 	if (dest)
 		groupwise_transfer_messages_to (folder, uids, dest, NULL, TRUE, ex);
 	else {
@@ -567,9 +573,7 @@ move_to_junk (CamelFolder *folder, CamelMessageInfo *info, CamelException *ex)
 		else
 			groupwise_transfer_messages_to (folder, uids, dest, NULL, TRUE, ex);
 	}
-
 	update_junk_list (folder->parent_store, info, ADD_JUNK_ENTRY);
-
 }
 
 /********************* back to folder functions*************************/
@@ -629,11 +633,21 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 			continue;
 		flags = camel_message_info_flags (info);	
 
-		if ((flags & CAMEL_MESSAGE_JUNK) && !(flags & CAMEL_GW_MESSAGE_JUNK)) /*marked a message junk*/
+		if ((flags & CAMEL_MESSAGE_JUNK) && strcmp(camel_folder_get_name(folder), JUNK_FOLDER)) { 
+			/*marked a message junk*/
 			move_to_junk (folder, info, ex);
+			camel_folder_summary_remove_uid (folder->summary, camel_message_info_uid(info));
+			camel_data_cache_remove (gw_folder->cache, "cache", camel_message_info_uid(info), ex);
+			continue;
+		}
 
-		else if ((flags & CAMEL_MESSAGE_JUNK) && (flags & CAMEL_GW_MESSAGE_JUNK)) /*message was marked as junk, now unjunk*/ 
+		if ((flags & CAMEL_GW_MESSAGE_NOJUNK) && !strcmp(camel_folder_get_name(folder), JUNK_FOLDER)) {
+			/*message was marked as junk, now unjunk*/ 
 			move_to_mailbox (folder, info, ex);
+			camel_folder_summary_remove_uid (folder->summary, camel_message_info_uid(info));
+			camel_data_cache_remove (gw_folder->cache, "cache", camel_message_info_uid(info), ex);
+			continue;
+		}
 
 		if (gw_info && (gw_info->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
 			do_flags_diff (&diff, gw_info->server_flags, gw_info->info.flags);
@@ -680,7 +694,9 @@ groupwise_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 		}
 		camel_message_info_free (info);
 	}
+	
 	CAMEL_GROUPWISE_FOLDER_UNLOCK (folder, cache_lock);
+
 	if (deleted_items) {
 		CAMEL_SERVICE_LOCK (gw_store, connect_lock);
 		if (!strcmp (folder->full_name, "Trash")) {
@@ -2361,7 +2377,7 @@ convert_to_calendar (EGwItem *item, char **str, int *len)
 	g_strfreev (tmp);
 }
 
-void 
+static void 
 convert_to_task (EGwItem *item, char **str, int *len)
 {
 	EGwItemOrganizer *org = NULL;
@@ -2427,4 +2443,39 @@ convert_to_task (EGwItem *item, char **str, int *len)
 	g_strfreev (tmp);
 }
 
+static void 
+convert_to_note (EGwItem *item, char **str, int *len)
+{
+	EGwItemOrganizer *org = NULL;
+	GString *gstr = g_string_new (NULL);
+	char **tmp = NULL;
+
+	tmp = g_strsplit (e_gw_item_get_id (item), "@", -1);
+
+	gstr = g_string_append (gstr, "BEGIN:VCALENDAR\n");
+	gstr = g_string_append (gstr, "METHOD:PUBLISH\n");
+	gstr = g_string_append (gstr, "BEGIN:VJOURNAL\n");
+	g_string_append_printf (gstr, "UID:%s\n",e_gw_item_get_icalid (item));
+	g_string_append_printf (gstr, "DTSTART:%s\n",e_gw_item_get_start_date (item));
+	g_string_append_printf (gstr, "SUMMARY:%s\n", e_gw_item_get_subject (item));
+	g_string_append_printf (gstr, "DESCRIPTION:%s\n", e_gw_item_get_message (item));
+	g_string_append_printf (gstr, "DTSTAMP:%s\n", e_gw_item_get_creation_date (item));
+	g_string_append_printf (gstr, "X-GWMESSAGEID:%s\n", e_gw_item_get_id (item));
+	g_string_append_printf (gstr, "X-GWRECORDID:%s\n", tmp[0]);
+
+	org = e_gw_item_get_organizer (item);
+	if (org)
+		g_string_append_printf (gstr, "ORGANIZER;CN= %s;ROLE= CHAIR;\n MAILTO:%s\n", 
+				org->display_name, org->email);
+	
+	gstr = g_string_append (gstr, "END:VJOURNAL\n");
+	gstr = g_string_append (gstr, "END:VCALENDAR\n");
+
+	*str = g_strdup (gstr->str);
+	*len = gstr->len;
+
+	g_string_free (gstr, TRUE);
+	g_strfreev (tmp);
+}
+
 /** End **/
